searchData={"items":[{"type":"module","title":"Memento","doc":"Simple + Powerful interface to the Erlang Mnesia Database.\n\n\nSee the [README](https://hexdocs.pm/memento) to get started.","ref":"Memento.html"},{"type":"function","title":"Memento.add_nodes/1","doc":"Tells Memento about other nodes running Memento/Mnesia.\n\nYou can use this to connect to and synchronize with other\nnodes at runtime and/or on discovery, to take full advantage\nof the distribution mode of Memento and Mnesia.\n\nThis is a wrapper method around `:mnesia.change_config/2`.","ref":"Memento.html#add_nodes/1"},{"type":"function","title":"Example - Memento.add_nodes/1","doc":"```\n# Connect to Memento running on a specific node\nMemento.add_nodes(:node_xyz@some_host)\n\n# Add all connected nodes to Memento distributed database\nMemento.add_nodes(Node.list())\n```","ref":"Memento.html#add_nodes/1-example"},{"type":"function","title":"Memento.info/0","doc":"Prints `:mnesia` information to console.","ref":"Memento.html#info/0"},{"type":"function","title":"Memento.start/0","doc":"Start the Memento Application.\n\nThis starts Memento and `:mnesia` along with some sane application\ndefaults. See `:mnesia.start/0` for more details.","ref":"Memento.html#start/0"},{"type":"function","title":"Memento.stop/0","doc":"Stop the Memento Application.","ref":"Memento.html#stop/0"},{"type":"function","title":"Memento.system/1","doc":"Returns all information about the Mnesia system.\n\nOptionally accepts a `key` atom argument which returns result for\nonly that key. Will throw an exception if that key is invalid. See\n`:mnesia.system_info/0` for more information and a full list of\nallowed keys.","ref":"Memento.html#system/1"},{"type":"function","title":"Memento.transaction/1","doc":"","ref":"Memento.html#transaction/1"},{"type":"function","title":"Memento.transaction!/1","doc":"","ref":"Memento.html#transaction!/1"},{"type":"function","title":"Memento.wait/1","doc":"","ref":"Memento.html#wait/1"},{"type":"function","title":"Memento.wait/2","doc":"","ref":"Memento.html#wait/2"},{"type":"module","title":"Memento.Query","doc":"Module to read/write from Memento Tables.\n\nThis module provides the most important transactional operations\nthat can be executed on Memento Tables. Mnesia's \"dirty\" methods\nare left out on purpose. In almost all circumstances, these\nmethods would be enough for interacting with Memento Tables, but\nfor very special situations, it is better to directly use the\nAPI provided by the Erlang `:mnesia` module.","ref":"Memento.Query.html"},{"type":"module","title":"Transaction Only - Memento.Query","doc":"All the methods exported by this module can only be executed\nwithin the context of a `Memento.Transaction`. Outside the\ntransaction (synchronous or not), these methods will raise an\nerror, even though they will be ignored in all examples moving\nforward.\n\n```\n# Will raise an error\nMemento.Query.read(Blog.Post, :some_id)\n\n# Will work fine\nMemento.transaction fn ->\n  Memento.Query.read(Blog.Post, :some_id)\nend\n```","ref":"Memento.Query.html#module-transaction-only"},{"type":"module","title":"Basic Operations - Memento.Query","doc":"```\n# Get all records in a Table\nMemento.Query.all(User)\n\n# Get a specific record by its primary key\nMemento.Query.read(User, id)\n\n# Write a record\nMemento.Query.write(%User{id: 3, name: \"Some User\"})\n\n# Delete a record by primary key\nMemento.Query.delete(User, id)\n\n# Delete a record by passing the full object\nMemento.Query.delete_record(%User{id: 4, name: \"Another User\"})\n```","ref":"Memento.Query.html#module-basic-operations"},{"type":"module","title":"Complex Queries - Memento.Query","doc":"Memento provides 3 ways of querying records based on some passed\nconditions:\n\n- `match/3`\n- `select/3`\n- `select_raw/3`\n\nEach method uses a different way of querying records, which is\nexplained in detail for each of them in their method docs. But\nthe recommended method of performing queries is using the\n`select/3` method, which makes working with Erlang MatchSpec a\nlot easier.\n\n```\n# Get all Movies\nMemento.Query.select(Movie, [])\n\n# Get all Movies named \"Rush\"\nMemento.Query.select(Movie, {:==, :title, \"Rush\"})\n\n# Get all Movies directed by Tarantino before the year 2000\nguards = [\n  {:==, :director, \"Quentin Tarantino\"},\n  {:<, :year, 2000},\n]\nMemento.Query.select(Movie, guards)\n```","ref":"Memento.Query.html#module-complex-queries"},{"type":"function","title":"Memento.Query.all/2","doc":"Returns all records of a Table.\n\nThis is equivalent to calling `match/3` with the catch-all pattern.\nThis also accepts an optional `lock` option to acquire that kind of\nlock in the transaction (defaults to `:read`). See `t:lock/0` for\nmore details about lock types.\n\n```\n# Both are equivalent\nMemento.Query.all(Movie)\nMemento.Query.match(Movie, {:_, :_, :_, :_})\n```","ref":"Memento.Query.html#all/2"},{"type":"function","title":"Memento.Query.delete/3","doc":"Delete a Record in the given table for the specified key.\n\nThis method takes a `Memento.Table` name and a key, and deletes all\nrecords with that key (There can be more than one for table type\nof `bag`). Options default to `[lock: :write]`.\n\nIf you want to delete a record, by passing the record itself as\nthe argument, see `delete_record/2`.","ref":"Memento.Query.html#delete/3"},{"type":"function","title":"Examples - Memento.Query.delete/3","doc":"```\n# Delete a Blog Post record with the id `10` (primary key)\nMemento.Query.delete(Blog.Post, 10)\n```","ref":"Memento.Query.html#delete/3-examples"},{"type":"function","title":"Memento.Query.delete_record/2","doc":"Delete the given Memento record object.\n\nThis method accepts a `t:Memento.Table.record/0` object and deletes\nthat from its table. A complete record object needs to be specified\nfor this to work. Options default to `[lock: :write]`.\n\nThis method is especially useful in Tables of type `bag` where\nmultiple records can have the same key. Also see `delete/3`.","ref":"Memento.Query.html#delete_record/2"},{"type":"function","title":"Examples - Memento.Query.delete_record/2","doc":"Consider an `Email` table of type `bag`, with two attributes;\n`user_id` and `email`, where `user_id` is the primary key. The Table\ncontains all email addresses for a given user.\n\n```\n# Calling `delete` will delete all emails for a `user_id`:\nMemento.Query.delete(Email, user_id)\n\n# To delete a specific record, you have to pass the entire object:\nemail_record = %Email{user_id: 5, email: \"a.specific@email.addr\"}\nMemento.Query.delete_record(email_record)\n```","ref":"Memento.Query.html#delete_record/2-examples"},{"type":"function","title":"Memento.Query.match/3","doc":"Returns all records in a table that match the specified pattern.\n\nThis method takes the name of a `Memento.Table` and a tuple pattern\nrepresenting the values of those attributes, and returns all\nrecords that match it. It uses `:_` to represent attributes that\nshould be ignored. The tuple passed should be of the same length as\nthe number of attributes in that table, otherwise it will throw an\nexception.\n\nIt's recommended to use the `select/3` method as it is more\nuser-friendly, can let you make complex selections.\n\nAlso accepts an optional argument `:lock` to acquire the kind of\nlock specified in that transaction (defaults to `:read`). See\n`t:lock/0` for more details. Also see `:mnesia.match_object/3`.","ref":"Memento.Query.html#match/3"},{"type":"function","title":"Examples - Memento.Query.match/3","doc":"Suppose a `Movie` Table with these attributes: `id`, `title`, `year`,\nand `director`. So the tuple passed in the match query should have\n4 elements.\n\n```\n# Get all movies from the Table\nMemento.Query.match(Movie, {:_, :_, :_, :_})\n\n# Get all movies named 'Rush', with a write lock on the item\nMemento.Query.match(Movie, {:_, \"Rush\", :_, :_}, lock: :write)\n\n# Get all movies directed by Tarantino\nMemento.Query.match(Movie, {:_, :_, :_, \"Quentin Tarantino\"})\n\n# Get all movies directed by Spielberg, in the year 1993\nMemento.Query.match(Movie, {:_, :_, 1993, \"Steven Spielberg\"})\n\n# Will raise exceptions\nMemento.Query.match(Movie, {:_, :_})\nMemento.Query.match(Movie, {:_, :_, :_})\nMemento.Query.match(Movie, {:_, :_, :_, :_, :_})\n```","ref":"Memento.Query.html#match/3-examples"},{"type":"function","title":"Memento.Query.read/3","doc":"Finds the Memento record for the given id in the specified table.\n\nIf no record is found, `nil` is returned. You can also pass an\noptional keyword list as the 3rd argument. The only option currently\nsupported is `:lock`, which acquires a lock of specified type on the\noperation (defaults to `:read`). See `t:lock/0` for more details.\n\nThis method works a bit differently from the original `:mnesia.read/3`\nwhen the table type is `:bag`. Since a bag can have many records\nwith the same key, this returns only the first one. If you want to\nfetch all records with the given key, use `match/3` or `select/3`.","ref":"Memento.Query.html#read/3"},{"type":"function","title":"Example - Memento.Query.read/3","doc":"```\nMemento.Query.read(Blog.Post, 1)\n# => %Blog.Post{id: 1, ... }\n\nMemento.Query.read(Blog.Post, 2, lock: :write)\n# => %Blog.Post{id: 2, ... }\n\nMemento.Query.read(Blog.Post, :unknown_id)\n# => nil\n```","ref":"Memento.Query.html#read/3-example"},{"type":"function","title":"Memento.Query.select/3","doc":"Return all records matching the given query.\n\nThis method takes a table name, and a simplified version of the Erlang\nMatchSpec consisting of one or more `guards`. Each guard is of the\nform `{function, argument_1, argument_2}`, where the arguments can be\nthe Table fields, literals or other nested guard functions.","ref":"Memento.Query.html#select/3"},{"type":"function","title":"Guard Spec - Memento.Query.select/3","doc":"Simple Operator Functions:\n\n- `:==` - Equality\n- `:===` - Strict Equality (For Numbers)\n- `:!=` - Inequality\n- `:!==` - Strict Inequality (For Numbers)\n- `:<` - Less than\n- `:<=` - Less than or equal to\n- `:>` - Greater than\n- `:>=` - Greater than or equal to\n\nGuard Functions that take nested arguments:\n\n- `:or`\n- `:and`\n- `:xor`","ref":"Memento.Query.html#select/3-guard-spec"},{"type":"function","title":"Options - Memento.Query.select/3","doc":"This method also takes some optional arguments mentioned below. See\n`t:options/0` for more details.\n\n- `lock` (defaults to `:read`)\n- `limit` (defaults to `nil`, meaning return all)\n- `coerce` (defaults to `true`)","ref":"Memento.Query.html#select/3-options"},{"type":"function","title":"Examples - Memento.Query.select/3","doc":"Suppose a `Movie` Table with these attributes: `id`, `title`, `year`,\nand `director`.\n\n```\n# Get all Movies\nMemento.Query.select(Movie, [])\n\n# Get all Movies named \"Rush\"\nMemento.Query.select(Movie, {:==, :title, \"Rush\"})\n\n# Get all Movies directed by Tarantino before the year 2000\n# Note: We could use a nested `and` function here as well\nguards = [\n  {:==, :director, \"Quentin Tarantino\"},\n  {:<, :year, 2000},\n]\nMemento.Query.select(Movie, guards)\n\n# Get all movies directed by Tarantino or Spielberg, in 2010 or later:\nguards =\n  {:and\n    {:>=, :year, 2010},\n    {:or,\n      {:==, :director, \"Quentin Tarantino\"},\n      {:==, :director, \"Steven Spielberg\"},\n    }\n  }\nMemento.Query.select(Movie, guards)\n```","ref":"Memento.Query.html#select/3-examples"},{"type":"function","title":"Memento.Query.select_raw/3","doc":"Returns all records in the given table according to the full Erlang\n`match_spec`.\n\nThis method accepts a pure Erlang `match_spec` term as described below,\nwhich can be used to write some very complex queries, but that also\nmakes it very hard to use for beginners, and overly complex for everyday\nqueries. It is highly recommended that you use the `select/3` method\nwhich makes it much easier to write complex queries that work just as\nwell in 99% of the cases, by making some assumptions.\n\nThe arguments are directly passed on to the `:mnesia.select/4` method\nwithout translating queries, as they are done in `select/3`.","ref":"Memento.Query.html#select_raw/3"},{"type":"function","title":"Options - Memento.Query.select_raw/3","doc":"See `t:options/0` for details about these options:\n\n- `lock` (defaults to `:read`)\n- `limit` (defaults to `nil`, meaning return all)\n- `coerce` (defaults to `true`)","ref":"Memento.Query.html#select_raw/3-options"},{"type":"function","title":"Match Spec - Memento.Query.select_raw/3","doc":"An Erlang \"Match Specification\" or `match_spec` is a term describing\na small program that tries to match something. This is most popularly\nused in both `:ets` and `:mnesia`. Quite simply, the grammar can be\ndefined as:\n\n- `match_spec` = `[match_function, ...]` (List of match functions)\n- `match_function` = `{match_head, [guard, ...], [result]}`\n- `match_head` = `tuple` (A tuple representing attributes to match)\n- `guard` = A tuple representing conditions for selection\n- `result` = Atom describing the fields to return as the result\n\nHere, the `match_head` describes the attributes to match (like in\n`match/3`). You can use literals to specify an exact value to be\nmatched against or `:\"$n\"` variables (`:$1`, `:$2`, ...)  that can be\nused so they can be referenced in the guards. You can get a default\nvalue by calling `YourTable.__info__().query_base`.\n\nThe second element in the tuple is a list of `guard` terms, where each\nguard is basically a tuple representing a condition of the form\n`{operation, arg1, arg2}` which can be simple `{:==, :\"$2\", literal}`\ntuples or nested values like `{:andalso, guard1, guard2}`. Finally,\n`result` represents the fields to return. Use `:\"$_\"` to return all\nfields, `:\"$n\"` to return a specific field or `:\"$$\"` for all fields\nspecified as variables in the `match_head`.","ref":"Memento.Query.html#select_raw/3-match-spec"},{"type":"function","title":"Examples - Memento.Query.select_raw/3","doc":"Suppose a `Movie` Table with these attributes: `id`, `title`, `year`,\nand `director`. So the tuple passed as the match_head should have\n5 elements.\n\nReturn all records:\n\n```\nmatch_head = Movie.__info__().query_base\nresult = [:\"$_\"]\nguards = []\n\nMemento.Query.select_raw(Movie, [{match_head, guards, result}])\n# => [%Movie{...}, ...]\n```\n\nGet all movies with the title \"Rush\":\n\n```\n# We're using the match_head pattern here, but you can also use guards\nmatch_head = {Movie, :\"$1\", \"Rush\", :\"$2\", :\"$3\"}\nresult = [:\"$_\"]\nguards = []\n\nMemento.Query.select_raw(Movie, [{match_head, guards, result}])\n# => [%Movie{title: \"Rush\", ...}, ...]\n```\n\nGet all movies title names, that were directed by Tarantino before the year 2000:\n\n```\n# Using guards only here, but you can mix and match with head.\n# You can also use a nested `{:andalso, guard1, guard2}` tuple\n# here instead.\n#\n# We used the result value `[:\"$2\"]` so it only returns the\n# second (i.e. title) field. Because of this, we're also not\n# coercing the results.\n\nmatch_head = {Movie, :\"$1\", :\"$2\", :\"$3\", :\"$4\"}\nresult = [:\"$2\"]\nguards = [{:<, :\"$3\", 2000}, {:==, :\"$4\", \"Quentin Tarantino\"}]\n\nMemento.Query.select_raw(Movie, [{match_head, guards, result}], coerce: false)\n# => [\"Reservoir Dogs\", \"Pulp Fiction\", ...]\n```\n\nGet all movies directed by Tarantino or Spielberg, after the year 2010:\n\n```\nmatch_head = {Movie, :\"$1\", :\"$2\", :\"$3\", :\"$4\"}\nresult = [:\"$_\"]\nguards = [\n  {:andalso,\n    {:>, :\"$3\", 2010},\n    {:orelse,\n      {:==, :\"$4\", \"Quentin Tarantino\"},\n      {:==, :\"$4\", \"Steven Spielberg\"},\n    }\n  }\n]\n\nMemento.Query.select_raw(Movie, [{match_head, guards, result}], coerce: true)\n# => [%Movie{...}, ...]\n```","ref":"Memento.Query.html#select_raw/3-examples"},{"type":"function","title":"Notes - Memento.Query.select_raw/3","doc":"- It's important to note that for customized results (not equal to\n`:\"$_\"`), you should specify `coerce: false`, so it doesn't raise errors.\n\n- Unlike the `select/3` method, the `operation` values the `guard` tuples\ntake in this method are Erlang atoms, not Elixir ones. For example,\ninstead of `:and` & `:or`, they will be `:andalso` & `:orelse`. Similarly,\nyou will have to use `:\"/=\"` instead of `:!=` and `:\"=<\"` instead of `:<=`.\n\nSee the [`Match Specification`](http://erlang.org/doc/apps/erts/match_spec.html)\ndocs, `:mnesia.select/2` and `:ets.select/2` more details and examples.","ref":"Memento.Query.html#select_raw/3-notes"},{"type":"function","title":"Memento.Query.write/2","doc":"Writes a Memento record to its Mnesia table.\n\nReturns the written record on success, or aborts the transaction\non failure. This operatiion acquires a lock of the kind specified,\nwhich can be either `:write` or `:sticky_write` (defaults to\n`:write`). See `t:lock/0` and `:mnesia.write/3` for more details.","ref":"Memento.Query.html#write/2"},{"type":"function","title":"Autoincrement and `nil` primary keys - Memento.Query.write/2","doc":"This method will raise an error if the primary key of the passed\nMemento record is `nil` and the table does not have autoincrement\nenabled. If it is enabled, this will find the last numeric key\nused, increment it and assign it as the primary key of the written\nrecord (which will be returned as a result of the write operation).\n\nTo enable autoincrement, the table needs to be of the type\n`ordered_set` and `autoincrement: true` has to be specified in\nthe table definition. (See `Memento.Table` for more details).","ref":"Memento.Query.html#write/2-autoincrement-and-nil-primary-keys"},{"type":"function","title":"Examples - Memento.Query.write/2","doc":"```\nMemento.Query.write(%Blog.Post{title: \"something\", ... })\n# => %Blog.Post{id: 4, title: \"something\"}\n\nMemento.Query.write(%Blog.Author{username: \"sye\", ... })\n# => %Blog.Author{username: \"sye\", ... }\n```","ref":"Memento.Query.html#write/2-examples"},{"type":"type","title":"Memento.Query.lock/0","doc":"Types of locks that can be acquired.\n\nThere are, in total, 3 types of locks that can be aqcuired, but\nsome operations don't support all of them. The `write/2` method,\nfor example, can only accept `:write` or `:sticky_write` locks.\n\nConflicting lock requests are automatically queued if there is\nno risk of deadlock. Otherwise, the transaction must be\nterminated and executed again. Memento does this automatically\nas long as the upper limit of `retries` is not reached in a\ntransaction.","ref":"Memento.Query.html#t:lock/0"},{"type":"type","title":"Types - Memento.Query.lock/0","doc":"- `:write` locks are exclusive. That means, if one transaction\nacquires a write lock, no other transaction can acquire any\nkind of lock on the same item.\n\n- `:read` locks can be shared, meaning if one transaction has a\nread lock on an item, other transactions can also acquire a\nread lock on the same item. However, no one else can acquire a\nwrite lock on that item while the read lock is in effect.\n\n- `:sticky_write` locks are used for optimizing write lock\nacquisitions, by informing other nodes which node is locked. New\nsticky lock requests from the same node are performed as local\noperations.\n\n\nFor more details, see `:mnesia.lock/2`.","ref":"Memento.Query.html#t:lock/0-types"},{"type":"type","title":"Memento.Query.options/0","doc":"Option Keyword that can be passed to some methods.\n\nThese are all the possible options that can be set in the given\nkeyword list, although it mostly depends on the method which\noptions it actually uses.","ref":"Memento.Query.html#t:options/0"},{"type":"type","title":"Options - Memento.Query.options/0","doc":"- `lock` - What kind of lock to acquire on the item in that\ntransaction. This is the most common option, that almost all\nmethods accept, and usually has some default value depending on\nthe method. See `t:lock/0` for more details.\n\n- `limit` - The maximum number of items to return in a query.\nThis is used only read queries like `match/3` or `select/3`, and\nis of the type `t:non_neg_integer/0`. Defaults to `nil`, resulting\nin no limit and returning all records.\n\n- `coerce` - Records in Mnesia are stored in the form of a Tuple.\nThis converts them into simple Memento struct records of type\n`t:Memento.Table.record/0`. This is equivalent to calling\n`Memento.Query.Data.load/1` on the returned records. This option is\nonly available to some read methods like `select/3` & `match/3`,\nand its value defaults to `true`.","ref":"Memento.Query.html#t:options/0-options"},{"type":"module","title":"Memento.Query.Data","doc":"Helper module that acts as an interface to convert Memento Table\ndata structs into Mnesia data tuples and vice versa.\n\nThis is responsible for automatically handling conversions\nbetween data in methods defined in the Query module, so you\nwon't need to use this at all.","ref":"Memento.Query.Data.html"},{"type":"module","title":"Usage - Memento.Query.Data","doc":"Given a Memento Table:\n\n```\ndefmodule MyApp.User do\n  use Memento.Table, attributes: [:id, :name]\nend\n```\n\nYou can convert its structs to Mnesia format:\n\n```\nMemento.Query.Data.dump(%MyApp.User{id: 1, name: \"Sye\"})\n# => {MyApp.User, 1, \"Sye\"}\n```\n\nOr convert it back to a struct:\n\n```\nMemento.Query.Data.load({MyApp.User, 2, \"Rick\"})\n# => %MyApp.User{id: 2, name: \"Rick\"}\n```","ref":"Memento.Query.Data.html#module-usage"},{"type":"function","title":"Memento.Query.Data.dump/1","doc":"Convert Memento Table struct into Mnesia data tuple.\n\nUse this method when writing data to an Mnesia database. The\nargument should be a struct of a previously defined Memento\ntable definition, and this will convert it into a tuple\nrepresenting an Mnesia record.","ref":"Memento.Query.Data.html#dump/1"},{"type":"function","title":"Memento.Query.Data.load/1","doc":"Convert Mnesia data tuple into Memento Table struct.\n\nUse this method when reading data from an Mnesia database. The\ndata should be in a tuple format, where the first element is\nthe Table name (`Memento.Table` definition).\n\nThis will automatically match the the tuple values against the\ntable's attributes and convert it into a struct of the Memento\ntable you defined.","ref":"Memento.Query.Data.html#load/1"},{"type":"module","title":"Memento.Schema","doc":"Module to interact with the database schema.\n\nFor persisting data, Mnesia databases need to be created on disk. This\nmodule provides an interface to create the database on the disk of the\nspecified nodes. Most of the time that is usually the node that the\napplication is running on.\n\n```\n# Create schema on current node\nMemento.Schema.create([ node() ]\n\n# Create schema on many nodes\nnode_list = [node(), :alice@host_x, :bob@host_y, :eve@host_z]\nMemento.Schema.create(node_list)\n```\n\nImportant thing to note here is that only the nodes where data has to\nbe persisted to disk have to be included. RAM-only nodes should be\nleft out. Disk schemas can also be deleted by calling `delete/1` and\nyou can get information about them by calling `info/0`.","ref":"Memento.Schema.html"},{"type":"module","title":"Example - Memento.Schema","doc":"```elixir\n# The nodes where you want to persist\nnodes = [ node() ]\n\n# Create the schema\nMemento.stop\nMemento.Schema.create(nodes)\nMemento.start\n\n# Create disc copies of your tables\nMemento.Table.create!(TableA, disc_copies: nodes)\nMemento.Table.create!(TableB, disc_copies: nodes)\n```","ref":"Memento.Schema.html#module-example"},{"type":"function","title":"Memento.Schema.create/1","doc":"Creates a new database on disk on the specified nodes.\n\nCalling `:mnesia.create_schema` for a custom path throws an exception\nif that path does not exist. Memento's version avoids this by ensuring\nthat the directory exists.\n\nAlso see `:mnesia.create_schema/1`.","ref":"Memento.Schema.html#create/1"},{"type":"function","title":"Memento.Schema.delete/1","doc":"Deletes the database previously created by `create/1` on the specified\nnodes.\n\nUse this with caution, as it makes persisting data obsolete. Also see\n`:mnesia.delete_schema/1`.","ref":"Memento.Schema.html#delete/1"},{"type":"function","title":"Memento.Schema.info/0","doc":"Prints schema information about all Tables to the console.","ref":"Memento.Schema.html#info/0"},{"type":"function","title":"Memento.Schema.info/1","doc":"Prints schema information about the specified Table to the console.","ref":"Memento.Schema.html#info/1"},{"type":"function","title":"Memento.Schema.set_storage_type/2","doc":"Sets the schema storage mode for the specified node.\n\nUseful when you want to change the schema mode on the fly,\nusually when connecting to a new, unsynchronized node on\ndiscovery at runtime.\n\nThe mode can only be `:ram_copies` or `:disc_copies`. If the\nstorage mode is set to `ram_copies`, then no table on that\nnode can be disc-resident.\n\nThis just calls `Memento.Table.set_storage_type/3` underneath\nwith `:schema` as the table. Also see\n`:mnesia.change_table_copy_type/3` for more details.","ref":"Memento.Schema.html#set_storage_type/2"},{"type":"function","title":"Example - Memento.Schema.set_storage_type/2","doc":"```\nMemento.Schema.set_storage_type(:node@host, :disc_copies)\n```","ref":"Memento.Schema.html#set_storage_type/2-example"},{"type":"behaviour","title":"Memento.Table","doc":"Defines a Memento Table schema for Mnesia","ref":"Memento.Table.html"},{"type":"behaviour","title":"Usage - Memento.Table","doc":"You can define an Mnesia Table by calling `use Memento.Table` with a few\noptions in your module.\n\n```\ndefmodule Blog.Post do\n  use Memento.Table, attributes: [:id, :title, :content]\nend\n```\n\nEach table then must be created before you can interact with it. You can do\nthat by calling `create/2`. It's usually a good idea to call this while\nyour application is being started:\n\n```\nMemento.Table.create(Blog.Post)\n```","ref":"Memento.Table.html#module-usage"},{"type":"behaviour","title":"Options - Memento.Table","doc":"The table definition and the `create/2` function both accept a keyword list\nspecifying the options for the table:\n\n- `attributes` - A required list of atoms representing the attribute names\n  of the records of the table. Must have at least two attributes, where the\n  first one is the primary key.\n\n- `type` - One of `:set`, `:ordered_set`, or `:bag`. Default is `:set`. In\n  a `:set`, all records have unique keys. In a `:bag`, several records can\n  have the same key, but the record content is unique. If a non-unique\n  record is stored, the old conflicting records are overwritten.\n\n- `index` - List of fields to index.\n\n- `autoincrement` - If the table is of the type `:ordered_set`, setting this\n  `true` will automatically assign numeric values to non-nil primary keys\n  when writing records (using `Memento.Query.write/2`). Will return an error\n  if the table is not of the type `:ordered_set`.\n\nThe only required option is `attributes`. See `:mnesia.create_table/2` for\na full list of options. See the following example that uses more options:\n\n\n```\ndefmodule Blog.Post do\n  use Memento.Table,\n    attributes: [:id, :title, :content, :status, :author_id],\n    index: [:status, :author_id],\n    type: :ordered_set,\n    autoincrement: true\n\n\n  # You can also define other methods\n  # or helper functions in the module\nend\n```","ref":"Memento.Table.html#module-options"},{"type":"callback","title":"Memento.Table.__info__/0","doc":"Returns Table definition information.\n\nEvery defined `Memento.Table` via the `use` macro, will export this\nmethod, returning information about its attributes, structure, options\nand other details.","ref":"Memento.Table.html#c:__info__/0"},{"type":"function","title":"Memento.Table.clear/1","doc":"Deletes all entries in the given Memento Table.\n\nReturns `:ok` on success and `{:error, reason}` on failure.","ref":"Memento.Table.html#clear/1"},{"type":"function","title":"Memento.Table.create/2","doc":"Creates a Memento Table for Mnesia.\n\nThis must be called before you can interact with the table in any way.\nUses the attributes specified in the table definition. Returns `:ok` on\nsuccess or `{:error, reason}` on failure. Will raise an error if the\npassed module isn't a Memento Table.\n\nYou can optionally pass a set of options keyword, which will override\nall options specified in the definition except `:attributes`.  See\n`:mnesia.create_table/2` for all available options.","ref":"Memento.Table.html#create/2"},{"type":"function","title":"Memento.Table.create!/2","doc":"Same as `create/2`, but raises error on failure.","ref":"Memento.Table.html#create!/2"},{"type":"function","title":"Memento.Table.create_copy/3","doc":"Makes a copy of a table at the given node.\n\nEspecially useful when you want to replicate a table on another\nnode on the fly, usually when connecting to it the first time.\n\nThe argument `type` must be a valid `storage_type()` atom. This\ncan also be used to create a replica of the internal `:schema`\ntable.\n\nAlso see `:mnesia.add_table_copy/3`.","ref":"Memento.Table.html#create_copy/3"},{"type":"function","title":"Example - Memento.Table.create_copy/3","doc":"```\n# Create an on-disc replica of `Users` table on another node\nMemento.Table.create_copy(Users, :some_node@host_x, :disc_copies)\n```","ref":"Memento.Table.html#create_copy/3-example"},{"type":"function","title":"Memento.Table.delete/1","doc":"Deletes a Memento Table for Mnesia.\n\nReturns `:ok` on success and `{:error, reason}` on failure.","ref":"Memento.Table.html#delete/1"},{"type":"function","title":"Memento.Table.delete!/1","doc":"Same as `delete/1`, but raises error on failure.","ref":"Memento.Table.html#delete!/1"},{"type":"function","title":"Memento.Table.delete_copy/2","doc":"Deletes the replica of a table on the specified node.\n\nWhen the last replica of a table is deleted, the table disappears\nentirely. This function can also be used to delete the replica of\nthe internal `:schema` table which will cause the Mnesia node to\nbe removed (Mnesia/Memento must be stopped first).\n\nAlso see `:mnesia.del_table_copy/2`.","ref":"Memento.Table.html#delete_copy/2"},{"type":"function","title":"Memento.Table.info/2","doc":"Returns all table information.\n\nOptionally accepts an extra atom argument `key` which returns result\nfor only that key. Will throw an exception if the key is invalid. See\n`:mnesia.table_info/2` for a full list of allowed keys.","ref":"Memento.Table.html#info/2"},{"type":"function","title":"Memento.Table.move_copy/3","doc":"Moves a table's copy from one node to the other.\n\nThis operation preserves the storage type of the table. For example,\na `:ram_copies` table when moved from one node, remains keeps its\n`:ram_copies` storage type on the new node.\n\nOther transactions can still read and write while it's being moved.\nThis function cannot be called on the internal `:local_content`\ntables.\n\nAlso see `:mnesia.move_table_copy/3`.","ref":"Memento.Table.html#move_copy/3"},{"type":"function","title":"Memento.Table.set_storage_type/3","doc":"Sets the storage type of a table for the specified node.\n\nUseful when you want to change the table's copy type on the fly,\nusually when connecting to a new, unsynchronized node on\ndiscovery at runtime.\n\nThe argument `type` must be a valid `storage_type()` atom. This\ncan also be used for the internal `:schema` table, but you should\nuse `Memento.Schema.set_storage_type/2` instead.\n\nSee `:mnesia.change_table_copy_type/3` for more details.","ref":"Memento.Table.html#set_storage_type/3"},{"type":"function","title":"Example - Memento.Table.set_storage_type/3","doc":"```\nMemento.Table.set_storage_type(MyTable, :node@host, :disc_copies)\n```","ref":"Memento.Table.html#set_storage_type/3-example"},{"type":"function","title":"Memento.Table.wait/2","doc":"Wait until specified tables are ready.\n\nBefore performing some tasks, it's necessary that certain tables\nare ready and accessible. This call hangs until all tables\nspecified are accessible, or until timeout is reached\n(default: 3000ms).\n\nThe `timeout` value can either be `:infinity` or an integer\nrepresenting time in milliseconds. If you pass a Table/Module that\ndoes not exist along with `:infinity` as timeout, it will hang your\nprocess until that table is created and ready.\n\nThis method can be accessed directly on the `Memento` module as well.\n\nFor more information, see `:mnesia.wait_for_tables/2`.","ref":"Memento.Table.html#wait/2"},{"type":"function","title":"Examples - Memento.Table.wait/2","doc":"```\n# Wait until the `Movies` table is ready\nMemento.Table.wait(Movies, :infinity)\n\n# Wait a maximum of 3 seconds until the two tables are ready\nMemento.wait([TableA, TableB])\n```","ref":"Memento.Table.html#wait/2-examples"},{"type":"type","title":"Memento.Table.name/0","doc":"A Memento.Table module","ref":"Memento.Table.html#t:name/0"},{"type":"type","title":"Memento.Table.record/0","doc":"A Memento.Table record data struct","ref":"Memento.Table.html#t:record/0"},{"type":"type","title":"Memento.Table.storage_type/0","doc":"Table storage/copy type","ref":"Memento.Table.html#t:storage_type/0"},{"type":"module","title":"Memento.Transaction","doc":"Memento's wrapper around Mnesia transactions. This module exports\nmethods `execute/2` and `execute_sync/2`, and their bang versions,\nwhich accept a function to be executed and an optional argument to\nset the maximum no. of retries until the transaction succeeds.\n\n`execute/1` and `execute!/1` can be directly called from the base\n`Memento` module as the alias `Memento.transaction/1` and\n`Memento.transaction!/1` methods, but if you want to specify a\ncustom `retries` value or use the synchronous version, you should\nuse the methods in this module.","ref":"Memento.Transaction.html"},{"type":"module","title":"Examples - Memento.Transaction","doc":"```\n# Read a User record\n{:ok, user} =\n  Memento.transaction fn ->\n    Memento.Query.read(User, id)\n  end\n\n# Get all Users, raising errors on aborts\nusers =\n  Memento.transaction! fn ->\n    Memento.Query.all(User)\n  end\n\n# Update a User record on all nodes synchronously,\n# with a maximum of 5 retries\noperation = fn ->\n  Memento.Query.write(%User{id: 3, name: \"New Value\"})\nend\nMemento.Transaction.execute_sync(operation, 5)\n```","ref":"Memento.Transaction.html#module-examples"},{"type":"function","title":"Memento.Transaction.abort/1","doc":"Aborts a Memento transaction.\n\nCauses the transaction to return an error tuple with the passed\nargument: `{:error, {:transaction_aborted, reason}}`. Outside\nthe context of a transaction, simply raises an error.\n\nIn the bang versions of the transactions, it raises a\n`Memento.TransactionAborted` error instead of returning the\nerror tuple. Default value for reason is `:no_reason_given`.","ref":"Memento.Transaction.html#abort/1"},{"type":"function","title":"Memento.Transaction.execute/2","doc":"Execute passed function as part of an Mnesia transaction.\n\nDefault value of `retries` is `:infinity`. Returns either\n`{:ok, result}` or `{:error, reason}`. Also see\n`:mnesia.transaction/2`.","ref":"Memento.Transaction.html#execute/2"},{"type":"function","title":"Memento.Transaction.execute!/2","doc":"Same as `execute/2` but returns the result or raises an error.","ref":"Memento.Transaction.html#execute!/2"},{"type":"function","title":"Memento.Transaction.execute_sync/2","doc":"Execute the transaction in synchronization with all nodes.\n\nThis method waits until the data has been committed and logged to\ndisk (if used) on all involved nodes before it finishes. This is\nuseful to ensure that a transaction process does not overload the\ndatabases on other nodes.\n\nReturns either `{:ok, result}` or `{:error, reason}`. Also see\n`:mnesia.sync_transaction/2`.","ref":"Memento.Transaction.html#execute_sync/2"},{"type":"function","title":"Memento.Transaction.execute_sync!/2","doc":"Same as `execute_sync/2` but returns the result or raises an error.","ref":"Memento.Transaction.html#execute_sync!/2"},{"type":"function","title":"Memento.Transaction.inside?/0","doc":"Checks if you are inside a transaction.","ref":"Memento.Transaction.html#inside?/0"},{"type":"type","title":"Memento.Transaction.retries/0","doc":"Maximum no. of retries for a transaction","ref":"Memento.Transaction.html#t:retries/0"},{"type":"extras","title":"Memento","doc":"<!-- Heading: Start -->\n \n   \n     \n   \n \n\n\n \n   \n     \n   \n   \n     \n   \n   \n     \n   \n   \n     \n   \n   \n     \n   \n \n\n \n   Simple but Powerful Elixir interface to the Erlang Mnesia Database  \n   Mnesia. Memento.  Get it?  \n \n\n \n<!-- Heading: End -->\n\n\n\n - 😀 **Easy to Use:** Provides a simple & intuitive API for working with [Mnesia][mnesia]\n - ⚡️ **Real-time:** Has extremely fast real-time data searches, even across many nodes\n - 💪 **Powerful Queries:** on top of Erlang's MatchSpec and QLC, that are much easier to use\n - 📓 **Detailed Documentation:** and examples for all methods on [HexDocs][docs]\n - 💾 **Persistent:** Schema can be coherently kept on disc & in memory\n - 🌐 **Distributed:** Data can easily be replicated on several nodes\n - 🌀 **Atomic:** A series of operations can be grouped in to a single atomic transaction\n - 🔍 **Focused:** Encourages good patterns by omitting dirty calls to the database\n - 🔧 **Mnesia Compatible:** You can still use Mnesia methods for Schemas and Tables created by Memento\n - ❄️  **No Dependencies:** Zero external dependencies; only uses the built-in Mnesia module\n - ⛅️ **MIT Licensed**: Free for personal and commercial use\n\n \n\n\n\n**Memento** is an extremely easy-to-use and powerful wrapper in Elixir that makes it intuitive to work with\n[Mnesia][mnesia], the Erlang Distributed Realtime Database. The original Mnesia API in Erlang is convoluted, unorganized\nand combined with the complex `MatchSpec` and `QLC` query language, is hard to work with in Elixir, especially for\nbeginners. Memento attempts to define a simple API to work with schemas, removing the majority of complexity associated\nwith it.","ref":"readme.html"},{"type":"extras","title":"Installation - Memento","doc":"Add `:memento` to your list of dependencies in your Mix file:\n\n```elixir\ndef deps do\n  [{:memento, \"~> 0.5.0\"}]\nend\n```\n\nIf your Elixir version is `1.3` or lower, also add it to your `applications` list:\n\n```elixir\ndef application do\n  [applications: [:memento]]\nend\n```\n\n_It's preferable to only add `:memento` and not `:mnesia` along with it._ This will ensure that that OTP calls to Mnesia\ngo through the Supervisor spec specified in Memento.","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Memento","doc":"It is highly recommended that a custom path to the Mnesia database location is specified, even on the local `:dev`\nenvironment (You can add `.mnesia` to your `.gitignore`):\n\n```elixir\n# config/config.exs\nconfig :mnesia,\n  dir: '.mnesia/#{Mix.env}/#{node()}'        # Notice the single quotes\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Usage - Memento","doc":"You start by defining a Module as a Memento Table by specifying its attributes, type and other options. At least two\nattributes are required, where the first one is the `primary-key` of the table. A simple definition looks like this:\n\n```elixir\ndefmodule Blog.Author do\n  use Memento.Table, attributes: [:username, :fullname]\nend\n```\n\nA slightly more complex definition that uses more options, could look like this:\n\n```elixir\ndefmodule Blog.Post do\n  use Memento.Table,\n    attributes: [:id, :title, :content, :status, :author],\n    index: [:status, :author],\n    type: :ordered_set,\n    autoincrement: true\n\n\n  # You can also define other methods\n  # or helper functions in the module\nend\n```\n\nOnce you have defined your schemas, you need to create them before you can interact with them:\n\n```elixir\nMemento.Table.create!(Blog.Author)\nMemento.Table.create!(Blog.Post)\n```\n\nSee the [`Memento.Table`][docs-table] documentation for detailed examples and more information about all the options.","ref":"readme.html#usage"},{"type":"extras","title":"CRUD Operations & Queries - Memento","doc":"Once a Table has been created, you can perform read/write/delete operations on their records. An API for all of\nthese operations is exposed in the [`Memento.Query`][docs-query] module, but these methods can't be called directly.\nInstead, they must always be called inside a [`Memento.Transaction`][docs-transaction]:\n\n```elixir\nMemento.transaction! fn ->\n  Memento.Query.all(Blog.Author)\nend\n\n# => [\n#  %Blog.Author{username: :sye,     fullname: \"Sheharyar Naseer\"},\n#  %Blog.Author{username: :jeanne,  fullname: \"Jeanne Bolding\"},\n#  %Blog.Author{username: :pshore,  fullname: \"Paul Shore\"},\n# ]\n```\n\n_For the sake of succinctness, transactions are ignored in most of the examples below, but they are still required._\nHere's a quick overview of all the basic operations:\n\n\n```elixir\n# Get all records in a Table\nMemento.Query.all(Post)\n\n# Get a specific record by its primary key\nMemento.Query.read(Post, id)\nMemento.Query.read(Author, username)\n\n# Write a record\nMemento.Query.write(%Author{username: :sarah, name: \"Sarah Molton\"})\n\n# Delete a record by primary key\nMemento.Query.delete(Post, id)\nMemento.Query.delete(Author, username)\n\n# Delete a record by passing the full object\nMemento.Query.delete_record(%Author{username: :pshore, name: \"Paul Shore\"})\n```\n\n\nFor more complex read operations, Memento exposes a [`select/3`][docs-query-select] method that lets you chain\nconditions using a simplified version of the Erlang MatchSpec. This is what some queries would look like for a\n`Movie` table:\n\n - Get all movies named \"Rush\":\n\n    ```elixir\n    Memento.Query.select(Movie, {:==, :title, \"Rush\"})\n    ```\n\n\n - Get all movies directed by Tarantino before the year 2000:\n\n    ```elixir\n    guards = [\n      {:==, :director, \"Quentin Tarantino\"},\n      {:<, :year, 2000},\n    ]\n    Memento.Query.select(Movie, guards)\n    ```\n\nSee [`Query.select/3`][docs-query-select] for more information about the guard operators and detailed examples.","ref":"readme.html#crud-operations-queries"},{"type":"extras","title":"Persisting to Disk - Memento","doc":"Setting up disk persistence in `Mnesia` has always been a bit weird. It involves stopping the application, creating\nschemas on disk, restarting the application and then creating the tables with certain options. Here are the steps\nyou need to take to do all of that:\n\n```elixir\n# List of nodes where you want to persist\nnodes = [ node() ]\n\n# Create the schema\nMemento.stop\nMemento.Schema.create(nodes)\nMemento.start\n\n# Create your tables with disc_copies (only the ones you want persisted on disk)\nMemento.Table.create!(TableA, disc_copies: nodes)\nMemento.Table.create!(TableB, disc_copies: nodes)\nMemento.Table.create!(TableC)\n```\n\nThis needs to be done only once and not every time the application starts. It also makes sense to create a helper\nfunction or mix task that does this for you. You can see a [sample implementation here][que-persistence].","ref":"readme.html#persisting-to-disk"},{"type":"extras","title":"Roadmap - Memento","doc":"- [x] Memento\n    - [x] start/stop\n    - [x] info\n    - [x] system_info\n    - [ ] Application\n    - [ ] Config Vars\n - [x] Memento.Table\n    - [x] Create/Delete helpers\n    - [x] clear_table\n    - [x] table_info\n    - [x] wait\n    - [ ] Ecto-like DSL\n    - [ ] Migration Support\n - [x] Memento.Query\n    - [x] Integration with Memento.Table\n    - [x] match/select\n    - [x] read/write/delete\n    - [ ] first/next/prev/all_keys\n    - [ ] test matchspec\n    - [ ] continue/1 for select continuations\n    - [x] autoincrement\n    - [ ] Helper use macro\n - [x] Memento.Transaction\n    - [x] Simple/Synchronous\n    - [x] Bang versions\n    - [x] inside?\n    - [x] abort\n    - [ ] Lock Helpers\n - [x] Memento.Schema\n    - [x] create/delete\n    - [x] print (schema/1)\n - [ ] Memento.Collection\n    - [ ] Easy Helpers\n    - [ ] Custom DSL\n  - [ ] Mix Tasks","ref":"readme.html#roadmap"},{"type":"extras","title":"FAQ - Memento","doc":"#### 1. Why Memento/Mnesia?\n\nIn most applications, some kind of data storage mechanism is needed, but this usually means relying on some sort\nof external dependency or program. Memento should be used in situations when it might not always make sense in an\nApplication to do this (e.g. the data is ephemeral, the project needs to be kept light-weight, you need a simple\ndata store that persists across application restarts, data-code decoupling is not important etc.).\n\n\n#### 2. When shouldn't I use Memento/Mnesia?\n\nLike mentioned in the previous point, Memento/Mnesia has specific use-cases and it might not always make sense to\nuse it. This is usually when you don't want to couple your code and database, and want to allow independent or\nexternal accesses to transformation of your data. In such circumstances, you should always prefer using some other\ndatastore (like Redis, Postgres, etc.).\n\n\n#### 3. Isn't there already an 'Amnesia' library?\n\nI've been a long-time user of the [`Amnesia`][amnesia] package, but with the recent releases of Elixir (1.5 & above),\nthe library has started to show its age. Amnesia's dependence on the the `exquisite` package has caused a lot of\ncompilation problems, and it's complex macro-intensive structure hasn't made it easy to fix them either. The library\nitself doesn't even compile in Elixir 1.7+ so I finally decided to write my own after I desperately needed to update\nmy Mnesia-based projects.\n\nMemento is meant to be an extremely lightweight wrapper for Mnesia, providing a very easy set of helpers and forcing\ngood decisions by avoiding the \"dirty\" methods.\n\n\n#### 4. Are there any other projects that are using Memento?\n\nMemento is a new package so there aren't many Open Source examples available. [Que][que] is another library\nthat uses Memento for background job processing and storing the state of these Jobs. If your project uses\nMemento, feel free to send in a pull-request so it can be mentioned here.","ref":"readme.html#faq"},{"type":"extras","title":"Contributing - Memento","doc":"- [Fork][github-fork], Enhance, Send PR\n - Lock issues with any bugs or feature requests\n - Implement something from Roadmap\n - Spread the word :heart:","ref":"readme.html#contributing"},{"type":"extras","title":"License - Memento","doc":"This package is available as open source under the terms of the [MIT License][license].\n\n \n\n\n\n\n\n  [logo]:                 media/logo.png\n  [shield-version]:       https://img.shields.io/hexpm/v/memento.svg\n  [shield-license]:       https://img.shields.io/hexpm/l/memento.svg\n  [shield-downloads]:     https://img.shields.io/hexpm/dt/memento.svg\n  [shield-travis]:        https://img.shields.io/travis/sheharyarn/memento/master.svg\n  [shield-inch]:          https://inch-ci.org/github/sheharyarn/memento.svg?branch=master\n\n  [travis-ci]:            https://travis-ci.org/sheharyarn/memento\n  [inch-ci]:              https://inch-ci.org/github/sheharyarn/memento\n\n  [license]:              ./LICENSE\n  [mnesia]:               http://erlang.org/doc/man/mnesia.html\n  [hexpm]:                https://hex.pm/packages/memento\n  [imdb-memento]:         https://www.imdb.com/title/tt0209144/\n  [que]:                  https://github.com/sheharyarn/que\n  [amnesia]:              https://github.com/meh/amnesia\n\n  [docs]:                 https://hexdocs.pm/memento\n  [docs-transaction]:     https://hexdocs.pm/memento/Memento.Transaction.html\n  [docs-table]:           https://hexdocs.pm/memento/Memento.Table.html\n  [docs-query]:           https://hexdocs.pm/memento/Memento.Query.html\n  [docs-query-select]:    https://hexdocs.pm/memento/Memento.Query.html#select/3\n\n  [github-fork]:          https://github.com/sheharyarn/memento/fork\n  [que-persistence]:      https://github.com/sheharyarn/que/blob/dc3764a27f8ce3e28b15a7bfafbca604fb424ecb/lib/que/persistence/mnesia/mnesia.ex#L90-L108","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"Changelog\n=========","ref":"changelog.html"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"- Patch release to fix hex publish issues\n - No functional changes in code","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"- Fix function warnings on recent Elixir versions\n - Update minimum Elixir version to 1.14","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"- [Bugfix] Convert mnesia's `:\"$end_of_table\"` code to empty list\n    - Previously raised UndefinedFunctionError during coercion","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"- [Bugfix] Catch Mnesia exits _outside_ transactions (#2)\n     - This fixes issues where Memento would unintentionally mess with Mnesia's\n       deadlock resolution algorithm by catching exits, finally causing Mnesia to\n       throw cyclic abort errors in high concurrency, nested transactions","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Before v0.3.0 - Changelog","doc":"Check the [commit history][commits].\n\n\n\n  [commits]: https://github.com/sheharyarn/memento/commits/master","ref":"changelog.html#before-v0-3-0"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2018 Sheharyar Naseer - https://shyr.io/\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}